@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Config;
 
-our $VERSION = '2.000015';
+our $VERSION = '2.000017';
 $VERSION = eval $VERSION;
 
 BEGIN {
@@ -85,6 +85,15 @@ sub _rel2abs {
   return _catdir($base, $dir);
 }
 
+our $_DEVNULL;
+sub _devnull {
+  return $_DEVNULL ||=
+    _USE_FSPEC      ? (require File::Spec, File::Spec->devnull)
+    : _WIN32        ? 'nul'
+    : $^O eq 'os2'  ? '/dev/nul'
+    : '/dev/null';
+}
+
 sub import {
   my ($class, @args) = @_;
   push @args, @ARGV
@@ -92,13 +101,14 @@ sub import {
 
   my @steps;
   my %opts;
+  my %attr;
   my $shelltype;
 
   while (@args) {
     my $arg = shift @args;
     # check for lethal dash first to stop processing before causing problems
     # the fancy dash is U+2212 or \xE2\x88\x92
-    if ($arg =~ /\xE2\x88\x92/ or $arg =~ /−/) {
+    if ($arg =~ /\xE2\x88\x92/) {
       die <<'DEATH';
 WHOA THERE! It looks like you've got some fancy dashes in your commandline!
 These are *not* the traditional -- dashes that software recognizes. You
@@ -134,18 +144,21 @@ DEATH
     elsif ( $arg eq '--no-create' ) {
       $opts{no_create} = 1;
     }
+    elsif ( $arg eq '--quiet' ) {
+      $attr{quiet} = 1;
+    }
     elsif ( $arg =~ /^--/ ) {
       die "Unknown import argument: $arg";
     }
     else {
-      push @steps, ['activate', $arg];
+      push @steps, ['activate', $arg, \%opts];
     }
   }
   if (!@steps) {
-    push @steps, ['activate', undef];
+    push @steps, ['activate', undef, \%opts];
   }
 
-  my $self = $class->new(%opts);
+  my $self = $class->new(%attr);
 
   for (@steps) {
     my ($method, @args) = @$_;
@@ -176,7 +189,7 @@ sub libs { $_[0]->{libs}   ||= [ \'PERL5LIB' ] }
 sub bins { $_[0]->{bins}   ||= [ \'PATH' ] }
 sub roots { $_[0]->{roots} ||= [ \'PERL_LOCAL_LIB_ROOT' ] }
 sub extra { $_[0]->{extra} ||= {} }
-sub no_create { $_[0]->{no_create} }
+sub quiet { $_[0]->{quiet} }
 
 my $_archname = $Config{archname};
 my $_version  = $Config{version};
@@ -317,11 +330,12 @@ sub deactivate_all {
 }
 
 sub activate {
-  my ($self, $path) = @_;
+  my ($self, $path, $opts) = @_;
+  $opts ||= {};
   $self = $self->new unless ref $self;
   $path = $self->resolve_path($path);
-  $self->ensure_dir_structure_for($path)
-    unless $self->no_create;
+  $self->ensure_dir_structure_for($path, { quiet => $self->quiet })
+    unless $opts->{no_create};
 
   $path = $self->normalize_path($path);
 
@@ -332,7 +346,7 @@ sub activate {
   }
 
   my %args;
-  if (!@active_lls || $active_lls[0] ne $path) {
+  if ($opts->{always} || !@active_lls || $active_lls[0] ne $path) {
     %args = (
       bins  => [ $self->install_base_bin_path($path), @{$self->bins} ],
       libs  => [ $self->install_base_perl_path($path), @{$self->libs} ],
@@ -354,11 +368,11 @@ sub normalize_path {
 }
 
 sub build_environment_vars_for {
-  my $self = $_[0]->new->activate($_[1]);
+  my $self = $_[0]->new->activate($_[1], { always => 1 });
   $self->build_environment_vars;
 }
 sub build_activate_environment_vars_for {
-  my $self = $_[0]->new->activate($_[1]);
+  my $self = $_[0]->new->activate($_[1], { always => 1 });
   $self->build_environment_vars;
 }
 sub build_deactivate_environment_vars_for {
@@ -418,7 +432,7 @@ sub print_environment_vars_for {
 }
 
 sub environment_vars_string_for {
-  my $self = $_[0]->new->activate($_[1]);
+  my $self = $_[0]->new->activate($_[1], { always => 1});
   $self->environment_vars_string;
 }
 sub environment_vars_string {
@@ -478,7 +492,7 @@ sub build_bourne_env_declaration {
 
 sub build_csh_env_declaration {
   my ($class, $name, $args) = @_;
-  my ($value, @vars) = $class->_interpolate($args, '${%s}', '"', '"\\%s"');
+  my ($value, @vars) = $class->_interpolate($args, '${%s}', qr/["\$]/, '"\\%s"');
   if (!defined $value) {
     return qq{unsetenv $name;\n};
   }
@@ -516,7 +530,7 @@ sub build_cmd_env_declaration {
 
 sub build_powershell_env_declaration {
   my ($class, $name, $args) = @_;
-  my $value = $class->_interpolate($args, '$env:%s', '"', '`%s');
+  my $value = $class->_interpolate($args, '$env:%s', qr/["\$]/, '`%s');
 
   if (!$value) {
     return qq{Remove-Item -ErrorAction 0 Env:\\$name;\n};
@@ -535,12 +549,25 @@ sub wrap_powershell_output {
 
 sub build_fish_env_declaration {
   my ($class, $name, $args) = @_;
-  my $value = $class->_interpolate($args, '$%s', qr/[\\"' ]/, '\\%s');
+  my $value = $class->_interpolate($args, '$%s', qr/[\\"'$ ]/, '\\%s');
   if (!defined $value) {
     return qq{set -e $name;\n};
   }
-  $value =~ s/$_path_sep/ /g;
-  qq{set -x $name $value;\n};
+
+  if ($name =~ /^(?:CD|MAN)?PATH$/) {
+    $value =~ s/$_path_sep/ /g;
+    my $silent = $name =~ /^(?:CD)?PATH$/ ? " ^"._devnull : '';
+    return qq{set -x $name $value$silent;\n};
+  }
+
+  my $out = '';
+  my $value_without = $value;
+  if ($value_without =~ s/(?:^|$_path_sep)\$$name(?:$_path_sep|$)//g) {
+    $out .= qq{set -q $name; and set -x $name $value;\n};
+    $out .= qq{set -q $name; or };
+  }
+  $out .= qq{set -x $name $value_without;\n};
+  $out;
 }
 
 sub _interpolate {
@@ -629,17 +656,26 @@ sub resolve_relative_path {
 }
 
 sub ensure_dir_structure_for {
-  my ($class, $path) = @_;
-  unless (-d $path) {
-    warn "Attempting to create directory ${path}\n";
+  my ($class, $path, $opts) = @_;
+  $opts ||= {};
+  my %dirs;
+  my @dirs = grep ! -d, (
+    $class->lib_paths_for($path),
+    $class->install_base_bin_path($path),
+  );
+  @dirs{@dirs} = ();
+  for my $dir (@dirs) {
+    require File::Basename;
+    my $parent = File::Basename::dirname($dir);
+    unless (exists $dirs{$parent} || -d $parent) {
+      $dirs{$parent} = undef;
+      push @dirs, $parent
+    }
   }
-  require File::Basename;
-  my @dirs;
-  while(!-d $path) {
-    push @dirs, $path;
-    $path = File::Basename::dirname($path);
-  }
-  mkdir $_ for reverse @dirs;
+  warn "Attempting to create directory ${path}\n"
+    if !$opts->{quiet} && @dirs;
+  mkdir $_ or die "Unable to create $_: $!"
+    for reverse @dirs;
   return;
 }
 
@@ -658,7 +694,7 @@ sub guess_shelltype {
   for ($shellbin) {
     return
         /csh$/                   ? 'csh'
-      : /fish/                   ? 'fish'
+      : /fish$/                  ? 'fish'
       : /command(?:\.com)?$/i    ? 'cmd'
       : /cmd(?:\.exe)?$/i        ? 'cmd'
       : /4nt(?:\.exe)?$/i        ? 'cmd'
@@ -701,9 +737,9 @@ From the shell -
   PATH="/home/username/perl5/bin:$PATH"; export PATH;
   PERL_LOCAL_LIB_ROOT="/home/usename/perl5:$PERL_LOCAL_LIB_ROOT"; export PERL_LOCAL_LIB_ROOT;
 
-From a .bashrc file -
+From a F<.bash_profile> or F<.bashrc> file -
 
-  [ $SHLVL -eq 1 ] && eval "$(perl -I$HOME/perl5/lib/perl5 -Mlocal::lib)"
+  eval "$(perl -I$HOME/perl5/lib/perl5 -Mlocal::lib)"
 
 =head2 The bootstrapping technique
 
@@ -757,7 +793,7 @@ starts using our newly generated lib/ directory. If you are using bash or
 any other Bourne shells, you can add this to your shell startup script this
 way:
 
-  echo '[ $SHLVL -eq 1 ] && eval "$(perl -I$HOME/perl5/lib/perl5 -Mlocal::lib)"' >>~/.bashrc
+  echo 'eval "$(perl -I$HOME/perl5/lib/perl5 -Mlocal::lib)"' >>~/.bashrc
 
 If you are using C shell, you can do this as follows:
 
@@ -770,7 +806,7 @@ If you passed to bootstrap a directory other than default, you also need to
 give that as import parameter to the call of the local::lib module like this
 way:
 
-  echo '[ $SHLVL -eq 1 ] && eval "$(perl -I$HOME/foo/lib/perl5 -Mlocal::lib=$HOME/foo)"' >>~/.bashrc
+  echo 'eval "$(perl -I$HOME/foo/lib/perl5 -Mlocal::lib=$HOME/foo)"' >>~/.bashrc
 
 After writing your shell configuration file, be sure to re-read it to get the
 changed settings into your current shell's environment. Bourne shells use
@@ -797,11 +833,13 @@ installation to install modules in different directories directly this way:
   cd ../mydir2
   ... REPEAT ...
 
-When used in a C<.bashrc> file, it is recommended that you protect against
-re-activating a directory in a sub-shell.  This can be done by checking the
-C<$SHLVL> variable as shown in synopsis.  Without this, sub-shells created by
-the user or other programs will override changes made to the parent shell's
-environment.
+If you use F<.bashrc> to activate a local::lib automatically, the local::lib
+will be re-enabled in any sub-shells used, overriding adjustments you may have
+made in the parent shell.  To avoid this, you can initialize the local::lib in
+F<.bash_profile> rather than F<.bashrc>, or protect the local::lib invocation
+with a C<$SHLVL> check:
+
+  [ $SHLVL -eq 1 ] && eval "$(perl -I$HOME/perl5/lib/perl5 -Mlocal::lib)"
 
 If you are working with several C<local::lib> environments, you may want to
 remove some of them from the current environment without disturbing the others.
@@ -969,8 +1007,8 @@ likely to cause issues on Win32 systems.
 
 =back
 
-Attempts to create the given path, and all required parent directories. Throws
-an exception on failure.
+Attempts to create a local::lib directory, including subdirectories and all
+required parent directories. Throws an exception on failure.
 
 =head2 print_environment_vars_for
 
